@@ -35,6 +35,8 @@ Program grant you additional permission to convey the resulting work.
 """
 
 import numpy as np
+from mpi4py import MPI
+import sys
 
 try:
     import matplotlib.pyplot as plt
@@ -115,7 +117,8 @@ grad = {}
 phase = -np.ones(nb_domain_grid_pts, dtype=int)
 for i, gradient in enumerate(gradients):
     rve = msp.Cell(nb_domain_grid_pts, domain_lengths,
-                   msp.Formulation.finite_strain, gradient)
+                   msp.Formulation.finite_strain, gradient,
+                   fft='mpi', communicator=MPI.COMM_WORLD)
     hard = msp.material.MaterialLinearElastic1_2d.make(
         rve, "hard", 1., .33)
     vacuum = msp.material.MaterialLinearElastic1_2d.make(
@@ -134,12 +137,12 @@ for i, gradient in enumerate(gradients):
         rve, applied_strain, solver, newton_tol=newton_tol,
         equil_tol=equil_tol, verbose=verbose)
     stress[i] = result.stress.reshape(
-        (dim, dim, len(gradient)//dim, *nb_domain_grid_pts), order='f')
+        (dim, dim, len(gradient)//dim, *rve.nb_subdomain_grid_pts), order='f')
     grad[i] = result.grad.reshape(
-        (dim, dim, len(gradient)//dim, *nb_domain_grid_pts), order='f')
+        (dim, dim, len(gradient)//dim, *rve.nb_subdomain_grid_pts), order='f')
 
 
-if matplotlib_found:
+if matplotlib_found and MPI.COMM_WORLD.Get_size() == 1:
     plt.figure()
 
     phase = np.stack((phase, phase), axis=0)
@@ -176,6 +179,17 @@ if matplotlib_found:
             plt.plot(g[0, 1, 0, center[0]+1, :], 'bx-')
 
     plt.tight_layout()
-    plt.show()
+
+    if len(sys.argv[:]) == 2:
+        if sys.argv[1] == 1:
+            print("I skip the ploting of the results because you gave '1' as "
+                  "first argument.")
+            pass
+    else:
+        plt.show()
+
 else:
-    print('Plotting disabled because matplotlib is not found.')
+    if MPI.COMM_WORLD.Get_size() != 1:
+        print('Plotting disabled because we are running MPI-parallel.')
+    else:
+        print('Plotting disabled because matplotlib is not found.')
