@@ -57,25 +57,25 @@ namespace muSpectre {
   Cell::Cell(Projection_ptr projection, SplitCell is_cell_split)
       : projection{std::move(projection)},
         fields{std::make_unique<muGrid::GlobalFieldCollection>(
-            this->get_spatial_dim())},
+            this->get_spatial_dim(),
+            this->projection->get_nb_subdomain_grid_pts(),
+            this->projection->get_subdomain_locations(),
+            muGrid::FieldCollection::SubPtMap_t{
+                {QuadPtTag, this->get_nb_quad_pts()},
+                {NodalPtTag, this->get_nb_nodal_pts()}})},
         // We request the DOFs for a single quadrature point, since quadrature
         // points are handled by the field.
         strain{this->fields->register_real_field(
             "strain",
-            dof_for_formulation(this->get_formulation(),
-                                this->get_material_dim(), OneQuadPt),
+            shape_for_formulation(this->get_formulation(),
+                                  this->get_material_dim()),
             QuadPtTag)},
         stress{this->fields->register_real_field(
             "stress",
-            dof_for_formulation(this->get_formulation(),
-                                this->get_material_dim(), OneQuadPt),
+            shape_for_formulation(this->get_formulation(),
+                                  this->get_material_dim()),
             QuadPtTag)},
-        is_cell_split{is_cell_split} {
-    this->fields->set_nb_sub_pts(QuadPtTag, this->get_nb_quad_pts());
-    this->fields->set_nb_sub_pts(NodalPtTag, this->get_nb_nodal_pts());
-    this->fields->initialise(this->projection->get_nb_subdomain_grid_pts(),
-                             this->projection->get_subdomain_locations());
-  }
+        is_cell_split{is_cell_split} {}
 
   /* ---------------------------------------------------------------------- */
   bool Cell::is_initialised() const { return this->initialised; }
@@ -126,11 +126,20 @@ namespace muSpectre {
 
   /* ---------------------------------------------------------------------- */
   void Cell::complete_material_assignment_simple(MaterialBase & material) {
-    for (auto && mat : this->materials) {
-      if (mat->get_name() != material.get_name()) {
-        mat->initialise();
+    if (this->is_initialised()) {
+      throw RuntimeError(
+          "The cell is already initialised. Therefore, it is not "
+          "possible to complete material assignemnt for it");
+    } else {
+      for (auto && mat : this->materials) {
+        if (mat->get_name() != material.get_name()) {
+          if (!mat->get_is_initialised()) {
+            mat->initialise();
+          }
+        }
       }
     }
+
     auto nb_pixels = muGrid::CcoordOps::get_size(
         this->get_projection().get_nb_subdomain_grid_pts());
     std::vector<bool> assignments(nb_pixels, false);
@@ -158,16 +167,16 @@ namespace muSpectre {
   }
 
   /* ---------------------------------------------------------------------- */
-  std::array<Index_t, 2> Cell::get_strain_shape() const {
+  Shape_t Cell::get_strain_shape() const {
     switch (this->get_formulation()) {
     case Formulation::finite_strain: {
-      return std::array<Index_t, 2>{this->get_material_dim(),
-                                    this->get_material_dim()};
+      return Shape_t{this->get_material_dim(),
+                     this->get_material_dim()};
       break;
     }
     case Formulation::small_strain: {
-      return std::array<Index_t, 2>{this->get_material_dim(),
-                                    this->get_material_dim()};
+      return Shape_t{this->get_material_dim(),
+                     this->get_material_dim()};
       break;
     }
     default:
@@ -205,7 +214,6 @@ namespace muSpectre {
     std::vector<MaterialBase *> assignments(nb_pixels, nullptr);
 
     for (auto & mat : this->materials) {
-      mat->initialise();
       for (auto & index : mat->get_pixel_indices()) {
         auto & assignment{assignments.at(index)};
         if (assignment != nullptr) {
@@ -240,14 +248,22 @@ namespace muSpectre {
   }
 
   /* ---------------------------------------------------------------------- */
-  void Cell::initialise(muFFT::FFT_PlanFlags flags) {
+  void Cell::initialise() {
     // check that all pixels have been assigned exactly one material
-    for (auto && mat : this->materials) {
-      mat->initialise();
+    if (this->is_initialised()) {
+      throw RuntimeError(
+          "The cell is already initialised. Therefore, it is not "
+          "possible to complete material assignemnt for it");
+    } else {
+      for (auto && mat : this->materials) {
+        if (!mat->get_is_initialised()) {
+          mat->initialise();
+        }
+      }
     }
     this->check_material_coverage();
     // initialise the projection and compute the fft plan
-    this->projection->initialise(flags);
+    this->projection->initialise();
     this->initialised = true;
   }
 
@@ -280,10 +296,8 @@ namespace muSpectre {
       if (do_create) {
         this->tangent = this->fields->register_real_field(
             "Tangent_stiffness",
-            muGrid::ipow(dof_for_formulation(this->get_formulation(),
-                                             this->get_material_dim(),
-                                             OneQuadPt),
-                         2),
+            t4shape_for_formulation(this->get_formulation(),
+                                    this->get_material_dim()),
             QuadPtTag);
       } else {
         throw RuntimeError("Tangent has not been created");
@@ -406,6 +420,7 @@ namespace muSpectre {
   /* ---------------------------------------------------------------------- */
   muGrid::GlobalFieldCollection & Cell::get_fields() { return *this->fields; }
 
+  using muGrid::operator<<;
   /* ---------------------------------------------------------------------- */
   void Cell::apply_projection(muGrid::TypedFieldBase<Real> & field) {
     this->projection->apply_projection(field);
@@ -439,11 +454,11 @@ namespace muSpectre {
       throw RuntimeError("evaluate_projected_directional_stiffness "
                          "requires the tangent moduli");
     }
-    if (delta_strain.get_nb_dof_per_sub_pt() != this->get_strain_size()) {
+    if (delta_strain.get_nb_components() != this->get_strain_size()) {
       std::stringstream err{};
       err << "The input field should have " << this->get_strain_size()
           << " components per quadrature point, but has "
-          << delta_strain.get_nb_dof_per_sub_pt() << " components.";
+          << delta_strain.get_nb_components() << " components.";
       throw RuntimeError(err.str());
     }
     if (delta_strain.get_nb_sub_pts() != this->get_strain().get_nb_sub_pts()) {
@@ -555,7 +570,7 @@ namespace muSpectre {
         auto && field{muGrid::TypedField<T>::safe_cast(
             collection.get_field(unique_name))};
         local_fields.push_back(field);
-        nb_component_categories.insert(field.get_nb_dof_per_sub_pt());
+        nb_component_categories.insert(field.get_nb_components());
         tag_categories.insert(field.get_sub_division_tag());
       }
     }
@@ -574,7 +589,7 @@ namespace muSpectre {
           auto & coll = mat->get_collection();
           if (coll.field_exists(unique_name)) {
             auto & field{coll.get_field(unique_name)};
-            err_str << field.get_nb_dof_per_sub_pt()
+            err_str << field.get_nb_components()
                     << " components in material '" << mat->get_name() << "'"
                     << std::endl;
           }
@@ -616,8 +631,12 @@ namespace muSpectre {
     const std::string tag{*tag_categories.begin()};
 
     // get and prepare the field
-    auto & global_field{this->fields->template register_field<T>(
-        unique_name, nb_components, tag)};
+    muGrid::TypedField<T> & global_field{
+        this->fields->field_exists(unique_name)
+            ? dynamic_cast<muGrid::TypedField<T> &>(
+                  this->fields->get_field(unique_name))
+            : this->fields->template register_field<T>(unique_name,
+                                                       nb_components, tag)};
     global_field.set_zero();
 
     auto global_map{global_field.get_pixel_map()};
@@ -638,7 +657,7 @@ namespace muSpectre {
   /* ---------------------------------------------------------------------- */
   template <typename T>
   muGrid::TypedField<T> &
-  Cell::globalise_old_field(const std::string & unique_name,
+  Cell::globalise_old_field(const std::string & unique_prefix,
                             const size_t & nb_steps_ago) {
     // start by checking that the field exists at least once, and that
     // it always has th same number of components
@@ -648,12 +667,12 @@ namespace muSpectre {
 
     for (auto & mat : this->materials) {
       auto && collection{mat->get_collection()};
-      if (collection.state_field_exists(unique_name)) {
-        auto && state_field{collection.get_state_field(unique_name)};
+      if (collection.state_field_exists(unique_prefix)) {
+        auto && state_field{collection.get_state_field(unique_prefix)};
         auto && field_old{
             muGrid::TypedField<T>::safe_cast(state_field.old(nb_steps_ago))};
         local_fields_old.push_back(field_old);
-        nb_component_categories.insert(field_old.get_nb_dof_per_sub_pt());
+        nb_component_categories.insert(field_old.get_nb_components());
         tag_categories.insert(field_old.get_sub_division_tag());
       }
     }
@@ -664,21 +683,23 @@ namespace muSpectre {
       std::stringstream err_str{};
       if (nb_match > 1) {
         err_str
-            << "The state fields named '" << unique_name << "' do not have the "
+            << "The state fields named '" << unique_prefix
+            << "' do not have the "
             << "same number of components in every material, which is a "
             << "requirement for globalising them! The following values were "
             << "found by material:" << std::endl;
         for (auto & mat : this->materials) {
           auto && coll{mat->get_collection()};
-          if (coll.state_field_exists(unique_name)) {
-            auto && field{coll.get_state_field(unique_name).old(nb_steps_ago)};
-            err_str << field.get_nb_dof_per_sub_pt()
+          if (coll.state_field_exists(unique_prefix)) {
+            auto && field{
+                coll.get_state_field(unique_prefix).old(nb_steps_ago)};
+            err_str << field.get_nb_components()
                     << " components in material '" << mat->get_name() << "'"
                     << std::endl;
           }
         }
       } else {
-        err_str << "The state field named '" << unique_name
+        err_str << "The state field named '" << unique_prefix
                 << " does not exist in "
                 << "any of the materials and can therefore not be globalised!";
       }
@@ -693,20 +714,21 @@ namespace muSpectre {
       std::stringstream err_str{};
       if (nb_match > 1) {
         err_str
-            << "The fields named '" << unique_name << "' do not have the "
+            << "The fields named '" << unique_prefix << "' do not have the "
             << "same sub-division in every material, which is a "
             << "requirement for globalising them! The following values were "
             << "found by material:" << std::endl;
         for (auto & mat : this->materials) {
           auto & coll = mat->get_collection();
-          if (coll.field_exists(unique_name)) {
-            auto & field{coll.get_field(unique_name)};
+          if (coll.field_exists(unique_prefix)) {
+            auto & field{coll.get_field(unique_prefix)};
             err_str << "tag '" << field.get_sub_division_tag()
                     << "' in material '" << mat->get_name() << "'" << std::endl;
           }
         }
       } else {
-        err_str << "The field named '" << unique_name << "' does not exist in "
+        err_str << "The field named '" << unique_prefix
+                << "' does not exist in "
                 << "any of the materials and can therefore not be globalised!";
       }
       throw RuntimeError(err_str.str());
@@ -714,8 +736,12 @@ namespace muSpectre {
 
     const std::string tag{*tag_categories.begin()};
 
-    auto & global_field{this->fields->template register_field<T>(
-        unique_name, nb_components, tag)};
+    muGrid::TypedField<T> & global_field{
+        this->fields->field_exists(unique_prefix)
+            ? dynamic_cast<muGrid::TypedField<T> &>(
+                  this->fields->get_field(unique_prefix))
+            : this->fields->template register_field<T>(unique_prefix,
+                                                       nb_components, tag)};
     global_field.set_zero();
 
     auto global_map{global_field.get_pixel_map()};
@@ -736,7 +762,7 @@ namespace muSpectre {
   /* ---------------------------------------------------------------------- */
   template <typename T>
   muGrid::TypedField<T> &
-  Cell::globalise_current_field(const std::string & unique_name) {
+  Cell::globalise_current_field(const std::string & unique_prefix) {
     // start by checking that the field exists at least once, and that
     // it always has th same number of components
     std::set<Index_t> nb_component_categories{};
@@ -746,12 +772,12 @@ namespace muSpectre {
         local_fields_current;
     for (auto & mat : this->materials) {
       auto && collection{mat->get_collection()};
-      if (collection.state_field_exists(unique_name)) {
-        auto && state_field{collection.get_state_field(unique_name)};
+      if (collection.state_field_exists(unique_prefix)) {
+        auto && state_field{collection.get_state_field(unique_prefix)};
         auto && field_current(
             muGrid::TypedField<T>::safe_cast(state_field.current()));
         local_fields_current.push_back(field_current);
-        nb_component_categories.insert(field_current.get_nb_dof_per_sub_pt());
+        nb_component_categories.insert(field_current.get_nb_components());
         tag_categories.insert(field_current.get_sub_division_tag());
       }
     }
@@ -762,21 +788,22 @@ namespace muSpectre {
       std::stringstream err_str{};
       if (nb_match > 1) {
         err_str
-            << "The state fields named '" << unique_name << "' do not have the "
+            << "The state fields named '" << unique_prefix
+            << "' do not have the "
             << "same number of components in every material, which is a "
             << "requirement for globalising them! The following values were "
             << "found by material:" << std::endl;
         for (auto & mat : this->materials) {
           auto && coll{mat->get_collection()};
-          if (coll.state_field_exists(unique_name)) {
-            auto && field{coll.get_state_field(unique_name).current()};
-            err_str << field.get_nb_dof_per_sub_pt()
+          if (coll.state_field_exists(unique_prefix)) {
+            auto && field{coll.get_state_field(unique_prefix).current()};
+            err_str << field.get_nb_components()
                     << " components in material '" << mat->get_name() << "'"
                     << std::endl;
           }
         }
       } else {
-        err_str << "The state field named '" << unique_name
+        err_str << "The state field named '" << unique_prefix
                 << "' does not exist in "
                 << "any of the materials and can therefore not be globalised!";
       }
@@ -791,28 +818,33 @@ namespace muSpectre {
       std::stringstream err_str{};
       if (nb_match > 1) {
         err_str
-            << "The fields named '" << unique_name << "' do not have the "
+            << "The fields named '" << unique_prefix << "' do not have the "
             << "same sub-division in every material, which is a "
             << "requirement for globalising them! The following values were "
             << "found by material:" << std::endl;
         for (auto & mat : this->materials) {
           auto & coll = mat->get_collection();
-          if (coll.field_exists(unique_name)) {
-            auto & field{coll.get_field(unique_name)};
+          if (coll.field_exists(unique_prefix)) {
+            auto & field{coll.get_field(unique_prefix)};
             err_str << "tag '" << field.get_sub_division_tag()
                     << "' in material '" << mat->get_name() << "'" << std::endl;
           }
         }
       } else {
-        err_str << "The field named '" << unique_name << "' does not exist in "
+        err_str << "The field named '" << unique_prefix
+                << "' does not exist in "
                 << "any of the materials and can therefore not be globalised!";
       }
       throw RuntimeError(err_str.str());
     }
     const std::string tag{*tag_categories.begin()};
 
-    auto & global_field{this->fields->template register_field<T>(
-        unique_name, nb_components, tag)};
+    muGrid::TypedField<T> & global_field{
+        this->fields->field_exists(unique_prefix)
+            ? dynamic_cast<muGrid::TypedField<T> &>(
+                  this->fields->get_field(unique_prefix))
+            : this->fields->template register_field<T>(unique_prefix,
+                                                       nb_components, tag)};
     global_field.set_zero();
 
     auto global_map{global_field.get_pixel_map()};
@@ -856,7 +888,7 @@ namespace muSpectre {
   }
 
   /* ---------------------------------------------------------------------- */
-  bool Cell::is_non_linear() const {
+  bool Cell::was_last_eval_non_linear() const {
     if (this->get_formulation() == Formulation::finite_strain) {
       return true;
     }
