@@ -46,8 +46,8 @@ namespace muSpectre {
   /* ---------------------------------------------------------------------- */
   KrylovSolverCG::KrylovSolverCG(Cell & cell, Real tol, Uint maxiter,
                                  Verbosity verbose)
-      : Parent(cell, tol, maxiter, verbose), r_k(cell.get_nb_dof()),
-        p_k(cell.get_nb_dof()), Ap_k(cell.get_nb_dof()),
+      : Parent(cell, tol, maxiter, verbose),
+        r_k(cell.get_nb_dof()), p_k(cell.get_nb_dof()), Ap_k(cell.get_nb_dof()),
         x_k(cell.get_nb_dof()) {}
 
   /* ---------------------------------------------------------------------- */
@@ -62,7 +62,7 @@ namespace muSpectre {
     this->r_k = this->cell.get_adaptor() * this->x_k - rhs;
 
     this->p_k = -this->r_k;
-    this->converged = false;
+    this->convergence = Convergence::DidNotConverge;
 
     Real rdr = comm.sum(r_k.squaredNorm());
     Real rhs_norm2 = comm.sum(rhs.squaredNorm());
@@ -79,7 +79,7 @@ namespace muSpectre {
           << std::endl;
       std::cout << "WARNING: ";
       std::cout << msg.str();
-      this->converged = true;
+      this->convergence = Convergence::ReachedTolerance;
       return Vector_map(this->x_k.data(), this->x_k.size());
     }
 
@@ -93,34 +93,39 @@ namespace muSpectre {
 
     size_t count_width{};  // for output formatting in verbose case
     if (verbose > Verbosity::Silent) {
-      count_width = size_t(std::log10(this->maxiter)) + 1;
+      count_width = static_cast<size_t>(std::log10(this->maxiter)) + 1;
     }
 
-    for (Uint i = 0; i < this->maxiter && rdr > rel_tol2;
-         ++i, ++this->counter) {
+    for (Uint i = 0;
+         i < this->maxiter && rdr > rel_tol2; ++i, ++this->counter) {
       this->Ap_k = this->cell.get_adaptor() * this->p_k;
 
-      Real alpha = rdr / comm.sum(this->p_k.dot(this->Ap_k));
+      Real pdAp{comm.sum(this->p_k.dot(this->Ap_k))};
+      if (pdAp <= 0) {
+        // Hessian is not positive definite
+        throw SolverError("Hessian is not positive definite");
+      }
+      Real alpha{rdr / pdAp};
 
       this->x_k += alpha * this->p_k;
       this->r_k += alpha * this->Ap_k;
 
-      Real new_rdr = comm.sum(this->r_k.dot(this->r_k));
-      Real beta = new_rdr / rdr;
+      Real new_rdr{comm.sum(this->r_k.dot(this->r_k))};
+      Real beta{new_rdr / rdr};
       rdr = new_rdr;
 
       if (verbose > Verbosity::Silent && comm.rank() == 0) {
         std::cout << "  at CG step " << std::setw(count_width) << i
                   << ": |r|/|b| = " << std::setw(15)
-                  << std::sqrt(rdr / rhs_norm2) << ", cg_tol = " << this->tol
-                  << std::endl;
+                  << std::sqrt(rdr / rhs_norm2) << ", cg_tol = "
+                  << this->tol << std::endl;
       }
 
       this->p_k = -this->r_k + beta * this->p_k;
     }
 
     if (rdr < rel_tol2) {
-      this->converged = true;
+      this->convergence = Convergence::ReachedTolerance;
     } else {
       std::stringstream err{};
       err << " After " << this->counter << " steps, the solver "
