@@ -47,7 +47,7 @@ class DerivativeCheck2d(unittest.TestCase):
         self.field[:] = np.random.random(self.nb_pts)
         self.fft = muFFT.FFT(self.nb_pts)
         self.nb_dof = 1
-        self.fft.initialise(self.nb_dof)
+        self.fft.create_plan(self.nb_dof)
         self.fourier_field = self.fft.register_fourier_space_field(
             "fft_workspace", self.nb_dof)
         self.fft.fft(self.field, self.fourier_field)
@@ -97,7 +97,7 @@ class DerivativeCheck2d(unittest.TestCase):
         self.fft.ifft(d * self.fourier_field, diff_field)
         diff_field *= self.fft.normalisation
         self.fft.ifft(1j*2*np.pi*q[0] * fourier_field_copy, ndiff)
-        ndiff *=  self.fft.normalisation
+        ndiff *= self.fft.normalisation
         nx, ny = self.nb_pts
         diff_field = np.squeeze(diff_field)
         ndiff = np.squeeze(ndiff)
@@ -334,7 +334,7 @@ class DerivativeCheck3d(unittest.TestCase):
         self.field = np.random.random(self.nb_pts)
         self.fft = muFFT.FFT(self.nb_pts)
         self.nb_dof = 1
-        self.fft.initialise(self.nb_dof)
+        self.fft.create_plan(self.nb_dof)
         self.fourier_field = self.fft.register_fourier_space_field(
             "fft_workspace", self.nb_dof)
         self.fft.fft(self.field, self.fourier_field)
