@@ -33,6 +33,9 @@
  * Program grant you additional permission to convey the resulting work.
  *
  */
+
+#include <random>
+
 #include <libmufft/fft_utils.hh>
 #include <libmufft/fft_engine_base.hh>
 
@@ -57,7 +60,6 @@ namespace muSpectre {
       ProjectionFixture<threeD, threeD, Squares<threeD>,
                         DiscreteGradient<threeD>,
                         ProjectionFiniteStrain<threeD>>,
-
       ProjectionFixture<twoD, twoD, Squares<twoD>, DiscreteGradient<twoD>,
                         ProjectionFiniteStrainFast<twoD>>,
       ProjectionFixture<
@@ -70,7 +72,7 @@ namespace muSpectre {
   /* ---------------------------------------------------------------------- */
   BOOST_FIXTURE_TEST_CASE_TEMPLATE(constructor_test, fix, fixlist, fix) {
     BOOST_CHECK_NO_THROW(
-        fix::projector.initialise(muFFT::FFT_PlanFlags::estimate));
+        fix::projector.initialise());
   }
 
   /* ---------------------------------------------------------------------- */
@@ -385,7 +387,7 @@ namespace muSpectre {
     using proj = ProjectionFiniteStrain<twoD>;
     auto nb_dof{2 * 2};
     auto engine = std::make_unique<Engine>(DynCcoord_t{2, 3});
-    engine->initialise(nb_dof, muFFT::FFT_PlanFlags::estimate);
+    engine->create_plan(nb_dof);
     BOOST_CHECK_THROW(proj(std::move(engine), DynRcoord_t{4.3, 4.3}),
                       std::runtime_error);
   }
@@ -439,19 +441,17 @@ namespace muSpectre {
     Rcoord_t<mdim> delta_x{(fix::projector.get_domain_lengths() /
                             fix::projector.get_nb_domain_grid_pts())
                                .template get<mdim>()};
-    // fill the displacement field
-    for (auto && tup : akantu::zip(
-             fields.get_pixels().template get_dimensioned_pixels<mdim>(),
-             disp)) {
-      auto & ccoord = std::get<0>(tup);
-      auto & d = std::get<1>(tup);
-      Vector vec = muGrid::CcoordOps::get_vector(ccoord, delta_x);
-      Vector devided;
-      for (int k = 0; k < dim; k++) {
-        devided[k] = vec[k] / fix::projector.get_nb_domain_grid_pts()[k];
+
+    // fill the displacement field with random numbers
+    std::random_device rd;
+    std::mt19937 gen(rd());
+    std::uniform_real_distribution<> dis(-1.0, 1.0);
+    for (auto && d : disp) {
+      Vector u;
+      for (int k{0}; k < dim; ++k) {
+        u[k] = dis(gen);
       }
-      d.row(0) =
-          1.0 / (2.0 * muGrid::pi) * (2.0 * muGrid::pi * devided.array()).sin();
+      d.row(0) = u;
     }
 
     BOOST_TEST_CHECKPOINT("displacement field filled");
@@ -481,7 +481,7 @@ namespace muSpectre {
 
     BOOST_TEST_CHECKPOINT("gradient field computed");
 
-    fix::projector.initialise(muFFT::FFT_PlanFlags::estimate);
+    fix::projector.initialise();
     fix::projector.apply_projection(f_var);
 
     BOOST_TEST_CHECKPOINT("projection applied");
@@ -548,7 +548,7 @@ namespace muSpectre {
       gt.row(0) = g.row(0);
     }
 
-    fix::projector.initialise(muFFT::FFT_PlanFlags::estimate);
+    fix::projector.initialise();
     // apply projection once; G:f_grad
     fix::projector.apply_projection(f_grad);
 
